"use strict";
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var invariant_1 = require("./invariant");
var axios_1 = require("axios");
var effects_1 = require("redux-saga/effects");
var utils_1 = require("./utils");
var SagaRunner_1 = require("./SagaRunner");
var BaseStore = /** @class */ (function () {
    function BaseStore(baseStoreConfig) {
        if (baseStoreConfig === void 0) { baseStoreConfig = {}; }
        if (!BaseStore.initialized) {
            BaseStore.init();
        }
        baseStoreConfig = Object.assign({ bindState: true }, baseStoreConfig);
        this.http = BaseStore.http;
        this.sagaRunner = baseStoreConfig.sagaRunner || BaseStore.sagaRunner;
        this.key = baseStoreConfig.key || utils_1.getRandomText(6);
        if (baseStoreConfig.key) {
            this.sagaRunner.registerStore(baseStoreConfig.key, this);
        }
        if (baseStoreConfig.apiResToState) {
            this.apiResToState = baseStoreConfig.apiResToState;
        }
        this.bindState = baseStoreConfig.bindState;
        this.dispatch = this.dispatch.bind(this);
        this.runSaga = this.runSaga.bind(this);
        this.runCallWithSaga = this.runCallWithSaga.bind(this);
        this.processDecoratedMethods();
    }
    /**
     * 初始化静态字段
     * @param {BaseStoreStaticConfig} baseStoreConfig
     */
    BaseStore.init = function (baseStoreConfig) {
        if (baseStoreConfig === void 0) { baseStoreConfig = {}; }
        BaseStore.http = axios_1.default.create(baseStoreConfig.axiosConfig || {});
        BaseStore.sagaRunner = new SagaRunner_1.default(baseStoreConfig.sagaOptions);
        BaseStore.initialized = true;
    };
    /**
     * 重置静态字段
     */
    BaseStore.reset = function () {
        BaseStore.http = void 0;
        BaseStore.sagaRunner = void 0;
        BaseStore.initialized = false;
    };
    /**
     * 派发一个action
     * @param {Action} action
     * @returns {Action}
     */
    BaseStore.prototype.dispatch = function (action) {
        return this.sagaRunner.dispatch(action);
    };
    /**
     * 执行Saga方法
     * @param {Saga} saga 要执行的saga方法
     * @param args        saga方法的参数列表
     * @returns {Task}    sagaTask
     */
    BaseStore.prototype.runSaga = function (saga) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        return this.sagaRunner.runSaga.apply(this, [saga].concat(args));
    };
    BaseStore.prototype.processDecoratedMethods = function () {
        var _this = this;
        var prototype = Object.getPrototypeOf(this);
        var membersDescriptor = utils_1.getClassMembersDescriptor(prototype);
        var runSagaFuncArr = [];
        membersDescriptor.forEach(function (i) {
            var name = i.name, descriptor = i.descriptor;
            if (!descriptor || typeof descriptor.value !== 'function') {
                return;
            }
            var func = _this[name];
            // 先把标记取出来，因为在bind之后，标记会丢失
            var $bind = func.$bind, $apiConfig = func.$apiConfig, $runSaga = func.$runSaga;
            if ($bind) {
                _this[name] = _this[name].bind(_this);
            }
            if ($apiConfig) {
                var defaultParams_1 = $apiConfig.defaultParams;
                if (defaultParams_1 !== void 0) {
                    _this[name] = function (params) {
                        params = Object.assign({}, defaultParams_1, params);
                        return func(params);
                    };
                }
                _this.runCallWithSaga(name, $apiConfig);
            }
            if ($runSaga) {
                runSagaFuncArr.push(_this[name]);
            }
        });
        if (runSagaFuncArr.length > 0) {
            // 使用setTimeout避免子类的构造函数没执行完就开始执行saga入口方法
            setTimeout(function () {
                _this.runSaga(function () {
                    return __generator(this, function (_a) {
                        switch (_a.label) {
                            case 0: return [4 /*yield*/, effects_1.all(runSagaFuncArr.map(function (i) { return effects_1.fork(i); }))];
                            case 1:
                                _a.sent();
                                return [2 /*return*/];
                        }
                    });
                });
            });
        }
    };
    BaseStore.prototype.runCallWithSaga = function (funcName, apiConfig) {
        var _this = this;
        var func = this[funcName];
        var asyncTypeName = apiConfig.asyncTypeName, bindState = apiConfig.bindState, axiosApi = apiConfig.axiosApi, defaultParams = apiConfig.defaultParams;
        var asyncType = this[asyncTypeName];
        invariant_1.default(utils_1.isAsyncType(asyncType), 'invalid asyncType: %s', JSON.stringify(asyncType));
        invariant_1.default(!bindState || bindState in this, 'cannot find state key "%s" in store or state hasn\'t initialized', bindState);
        var that = this;
        var hasBindState = bindState && (bindState in this) && this.bindState;
        this[funcName] = function (params) {
            params = Object.assign({}, defaultParams, params);
            return _this.runSaga(function () {
                var data, _a, err_1;
                return __generator(this, function (_b) {
                    switch (_b.label) {
                        case 0:
                            if (!hasBindState) return [3 /*break*/, 2];
                            return [4 /*yield*/, utils_1.runInAction(function () {
                                    that[bindState].loading = true;
                                    that[bindState].error = null;
                                })];
                        case 1:
                            _b.sent();
                            _b.label = 2;
                        case 2: return [4 /*yield*/, effects_1.put({ type: asyncType.START, payload: params })];
                        case 3:
                            _b.sent();
                            data = null;
                            _b.label = 4;
                        case 4:
                            _b.trys.push([4, 11, , 15]);
                            return [4 /*yield*/, effects_1.call(func, params)];
                        case 5:
                            data = _b.sent();
                            if (!(that.apiResToState && axiosApi)) return [3 /*break*/, 7];
                            _a = data;
                            return [4 /*yield*/, effects_1.call(that.apiResToState, data.data)];
                        case 6:
                            _a.data = _b.sent();
                            _b.label = 7;
                        case 7:
                            if (!hasBindState) return [3 /*break*/, 9];
                            return [4 /*yield*/, utils_1.runInAction(function () {
                                    that[bindState].loading = false;
                                    if (axiosApi) {
                                        that[bindState].data = data.data;
                                    }
                                    else {
                                        that[bindState].data = data;
                                    }
                                })];
                        case 8:
                            _b.sent();
                            _b.label = 9;
                        case 9: return [4 /*yield*/, effects_1.put({ type: asyncType.END, payload: data })];
                        case 10:
                            _b.sent();
                            return [3 /*break*/, 15];
                        case 11:
                            err_1 = _b.sent();
                            if (!hasBindState) return [3 /*break*/, 13];
                            return [4 /*yield*/, utils_1.runInAction(function () {
                                    that[bindState].loading = false;
                                    that[bindState].error = err_1;
                                })];
                        case 12:
                            _b.sent();
                            _b.label = 13;
                        case 13: return [4 /*yield*/, effects_1.put({ type: asyncType.ERROR, payload: err_1 })];
                        case 14:
                            _b.sent();
                            console.error(err_1);
                            return [3 /*break*/, 15];
                        case 15: return [2 /*return*/, data];
                    }
                });
            }).done;
        };
    };
    /**
     * 静态对象是否已进行初始化
     * @type {boolean}
     */
    BaseStore.initialized = false;
    return BaseStore;
}());
exports.default = BaseStore;
//# sourceMappingURL=BaseStore.js.map